"""
Constants used throughout the TonieToolbox package
"""
SAMPLE_RATE_KHZ: int = 48
ONLY_CONVERT_FRAMEPACKING: int = -1
OTHER_PACKET_NEEDED: int = -2
DO_NOTHING: int = -3
TOO_MANY_SEGMENTS: int = -4
TIMESTAMP_DEDUCT: int = 0x50000000
OPUS_TAGS: list[bytearray] = [
    bytearray(
        b"\x4F\x70\x75\x73\x54\x61\x67\x73\x0D\x00\x00\x00\x4C\x61\x76\x66\x35\x38\x2E\x32\x30\x2E\x31\x30\x30\x03\x00\x00\x00\x26\x00\x00\x00\x65\x6E\x63\x6F\x64\x65\x72\x3D\x6F\x70\x75\x73\x65\x6E\x63\x20\x66\x72\x6F\x6D\x20\x6F\x70\x75\x73\x2D\x74\x6F\x6F\x6C\x73\x20\x30\x2E\x31\x2E\x31\x30\x2A\x00\x00\x00\x65\x6E\x63\x6F\x64\x65\x72\x5F\x6F\x70\x74\x69\x6F\x6E\x73\x3D\x2D\x2D\x71\x75\x69\x65\x74\x20\x2D\x2D\x62\x69\x74\x72\x61\x74\x65\x20\x39\x36\x20\x2D\x2D\x76\x62\x72\x3B\x01\x00\x00\x70\x61\x64\x3D\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30"),
    bytearray(
        b"\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30\x30")
]

# Mapping of language tags to ISO codes
LANGUAGE_MAPPING: dict[str, str] = {
    # Common language names to ISO codes
    'deutsch': 'de-de',
    'german': 'de-de',
    'english': 'en-us',
    'englisch': 'en-us',
    'français': 'fr-fr',
    'french': 'fr-fr',
    'franzosisch': 'fr-fr',
    'italiano': 'it-it',
    'italian': 'it-it',
    'italienisch': 'it-it',
    'español': 'es-es',
    'spanish': 'es-es',
    'spanisch': 'es-es',
    # Two-letter codes
    'de': 'de-de',
    'en': 'en-us',
    'fr': 'fr-fr',
    'it': 'it-it',
    'es': 'es-es',
}

# Mapping of genre tags to tonie categories
GENRE_MAPPING: dict[str, str] = {
    # Standard Tonie category names from tonies.json
    'hörspiel': 'Hörspiele & Hörbücher',
    'hörbuch': 'Hörspiele & Hörbücher',
    'hörbücher': 'Hörspiele & Hörbücher',
    'hörspiele': 'Hörspiele & Hörbücher',
    'audiobook': 'Hörspiele & Hörbücher',
    'audio book': 'Hörspiele & Hörbücher',
    'audio play': 'Hörspiele & Hörbücher',
    'audio-play': 'Hörspiele & Hörbücher',
    'audiospiel': 'Hörspiele & Hörbücher',
    'geschichte': 'Hörspiele & Hörbücher',
    'geschichten': 'Hörspiele & Hörbücher',
    'erzählung': 'Hörspiele & Hörbücher',
    
    # Music related genres
    'musik': 'music',
    'lieder': 'music',
    'songs': 'music',
    'music': 'music',
    'lied': 'music',
    'song': 'music',
    
    # More specific categories
    'kinder': 'Hörspiele & Hörbücher',
    'children': 'Hörspiele & Hörbücher',
    'märchen': 'Hörspiele & Hörbücher',
    'fairy tale': 'Hörspiele & Hörbücher',
    'märche': 'Hörspiele & Hörbücher',
    
    'wissen': 'Wissen & Hörmagazine',
    'knowledge': 'Wissen & Hörmagazine',
    'sachbuch': 'Wissen & Hörmagazine',
    'learning': 'Wissen & Hörmagazine',
    'educational': 'Wissen & Hörmagazine',
    'bildung': 'Wissen & Hörmagazine',
    'information': 'Wissen & Hörmagazine',
    
    'schlaf': 'Schlaflieder & Entspannung',
    'sleep': 'Schlaflieder & Entspannung',
    'meditation': 'Schlaflieder & Entspannung',
    'entspannung': 'Schlaflieder & Entspannung',
    'relaxation': 'Schlaflieder & Entspannung',
    'schlaflied': 'Schlaflieder & Entspannung',
    'einschlafhilfe': 'Schlaflieder & Entspannung',
    
    # Default to standard format for custom
    'custom': 'Hörspiele & Hörbücher',
}

    # Supported file extensions for audio files
SUPPORTED_EXTENSIONS = [
        '.wav', '.mp3', '.aac', '.m4a', '.flac', '.ogg', '.opus',
        '.ape', '.wma', '.aiff', '.mp2', '.mp4', '.webm', '.mka'
    ]

UTI_MAPPINGS = {
            'mp3': 'public.mp3',
            'wav': 'public.wav',
            'flac': 'org.xiph.flac',
            'ogg': 'org.xiph.ogg',
            'opus': 'public.opus',
            'aac': 'public.aac-audio',
            'm4a': 'public.m4a-audio',
            'wma': 'com.microsoft.windows-media-wma',
            'aiff': 'public.aiff-audio',
            'mp2': 'public.mp2',
            'mp4': 'public.mpeg-4-audio',
            'mka': 'public.audio',
            'webm': 'public.webm-audio',
            'ape': 'public.audio',
            'taf': 'public.audio'
        }

ARTWORK_NAMES = [
        'cover', 'folder', 'album', 'front', 'artwork', 'image', 
        'albumart', 'albumartwork', 'booklet'
    ]
ARTWORK_EXTENSIONS = ['.jpg', '.jpeg', '.png', '.bmp', '.gif']


TAG_VALUE_REPLACEMENTS = {
    "Die drei ???": "Die drei Fragezeichen",
    "Die Drei ???": "Die drei Fragezeichen",
    "DIE DREI ???": "Die drei Fragezeichen",
    "Die drei !!!": "Die drei Ausrufezeichen",
    "Die Drei !!!": "Die drei Ausrufezeichen",
    "DIE DREI !!!": "Die drei Ausrufezeichen",
    "TKKG™": "TKKG",
    "Die drei ??? Kids": "Die drei Fragezeichen Kids",
    "Die Drei ??? Kids": "Die drei Fragezeichen Kids",
    "Bibi & Tina": "Bibi und Tina",
    "Benjamin Blümchen™": "Benjamin Blümchen",
    "???": "Fragezeichen",
    "!!!": "Ausrufezeichen",
}

TAG_MAPPINGS = {
    # ID3 (MP3) tags
    'TIT2': 'title',
    'TALB': 'album',
    'TPE1': 'artist',
    'TPE2': 'albumartist',
    'TCOM': 'composer',
    'TRCK': 'tracknumber',
    'TPOS': 'discnumber',
    'TDRC': 'date',
    'TCON': 'genre',
    'TPUB': 'publisher',
    'TCOP': 'copyright',
    'COMM': 'comment',
    
    # Vorbis tags (FLAC, OGG)
    'title': 'title',
    'album': 'album',
    'artist': 'artist',
    'albumartist': 'albumartist',
    'composer': 'composer',
    'tracknumber': 'tracknumber',
    'discnumber': 'discnumber',
    'date': 'date',
    'genre': 'genre',
    'publisher': 'publisher',
    'copyright': 'copyright',
    'comment': 'comment',
    
    # MP4 (M4A, AAC) tags
    '©nam': 'title',
    '©alb': 'album',
    '©ART': 'artist',
    'aART': 'albumartist',
    '©wrt': 'composer',
    'trkn': 'tracknumber',
    'disk': 'discnumber',
    '©day': 'date',
    '©gen': 'genre',
    '©pub': 'publisher',
    'cprt': 'copyright',
    '©cmt': 'comment',
    
    # Additional tags some files might have
    'album_artist': 'albumartist',
    'track': 'tracknumber',
    'track_number': 'tracknumber',
    'disc': 'discnumber',
    'disc_number': 'discnumber',
    'year': 'date',
    'albuminterpret': 'albumartist',  # German tag name
    'interpret': 'artist',            # German tag name

}

CONFIG_TEMPLATE = {
    "metadata": {
        "description": "TonieToolbox configuration",
        "config_version": "1.0"     
    },
    "log_level": "silent", # Options: trace, debug, info, warning, error, critical, silent
    "log_to_file": False, # True if you want to log to a file ~\.tonietoolbox\logs
    "upload": {
        "url": [""], # https://teddycloud.example.com        
        "ignore_ssl_verify": False, # True if you want to ignore SSL certificate verification
        "username": "", # Basic Auth username
        "password": "", # Basic Auth password
        "client_cert_path": "", # Path to client certificate file
        "client_cert_key_path": "" # Path to client certificate key file
    }
}


# filepath: d:\Repository\TonieToolbox\TonieToolbox\integration_windows.py
import os
import sys
import json
from .constants import SUPPORTED_EXTENSIONS, CONFIG_TEMPLATE
from .logger import get_logger

logger = get_logger('integration_windows')

class WindowsClassicContextMenuIntegration:
    """
    Class to generate Windows registry entries for TonieToolbox "classic" context menu integration.
    Adds a 'TonieToolbox' cascade menu for supported audio files, .taf files, and folders.
    """
    def __init__(self):
        self.exe_path = os.path.join(sys.prefix, 'Scripts', 'tonietoolbox.exe')
        self.exe_path_reg = self.exe_path.replace('\\', r'\\')
        self.output_dir = os.path.join(os.path.expanduser('~'), '.tonietoolbox')
        self.icon_path = os.path.join(self.output_dir, 'icon.ico').replace('\\', r'\\')
        self.cascade_name = 'TonieToolbox'
        self.entry_is_separator = '"CommandFlags"=dword:00000008'
        self.show_uac = '"CommandFlags"=dword:00000010'
        self.separator_below = '"CommandFlags"=dword:00000040'
        self.separator_above = '"CommandFlags"=dword:00000020'
        self.error_handling = r' && if %ERRORLEVEL% neq 0 (echo Error: Command failed with error code %ERRORLEVEL% && pause && exit /b %ERRORLEVEL%) else (echo Command completed successfully && ping -n 2 127.0.0.1 > nul)'
        self.show_info_error_handling = r' && if %ERRORLEVEL% neq 0 (echo Error: Command failed with error code %ERRORLEVEL% && pause && exit /b %ERRORLEVEL%) else (echo. && echo Press any key to close this window... && pause > nul)'
        self.config = self._apply_config_template()
        self.upload_url = ''
        self.log_level = self.config.get('log_level', 'SILENT')
        self.log_to_file = self.config.get('log_to_file', False)
        self.basic_authentication_cmd = ''
        self.client_cert_cmd = ''
        self.upload_enabled = self._setup_upload()
        
        print(f"Upload enabled: {self.upload_enabled}")
        print(f"Upload URL: {self.upload_url}")
        print(f"Authentication: {'Basic Authentication' if self.basic_authentication else ('None' if self.none_authentication else ('Client Cert' if self.client_cert_authentication else 'Unknown'))}")
        
        self._setup_commands()

    def _build_cmd(self, base_args, file_placeholder='%1', output_to_source=True ,use_upload=False, use_artwork=False, use_json=False, use_compare=False, use_info=False, is_recursive=False, is_split=False, is_folder=False, shell='cmd.exe', keep_open=False, log_to_file=False):
        """Dynamically build command strings for registry entries."""
        exe = self.exe_path_reg        
        cmd = f'{shell} /{"k" if keep_open else "c"} "echo Running TonieToolbox'
        if use_info:
            cmd += ' info'
        elif is_split:
            cmd += ' split'
        elif use_compare:
            cmd += ' compare'
        elif is_recursive:
            cmd += ' recursive folder convert'
        elif is_folder:
            cmd += ' folder convert'
        elif use_upload and use_artwork and use_json:
            cmd += ' convert, upload, artwork and JSON'
        elif use_upload and use_artwork:
            cmd += ' convert, upload and artwork'
        elif use_upload:
            cmd += ' convert and upload'
        else:
            cmd += ' convert'
        cmd += ' command... && "'
        cmd += f'{exe}" {base_args}'
        if log_to_file:
            cmd += ' --log-file'
        if is_recursive:
            cmd += ' --recursive'
        if output_to_source:
            cmd += ' --output-to-source'
        if use_info:
            cmd += ' --info'
        if is_split:
            cmd += ' --split'
        if use_compare:
            cmd += ' --compare "%1" "%2"'
        else:
            cmd += f' "{file_placeholder}"'
        if use_upload:
            cmd += f' --upload "{self.upload_url}"'
            if self.basic_authentication_cmd:
                cmd += f' {self.basic_authentication_cmd}'
            elif self.client_cert_cmd:
                cmd += f' {self.client_cert_cmd}'
            if getattr(self, "ignore_ssl_verify", False):
                cmd += ' --ignore-ssl-verify'
        if use_artwork:
            cmd += ' --include-artwork'
        if use_json:
            cmd += ' --create-custom-json'
        if use_info or use_compare:
            cmd += ' && echo. && pause && exit > nul"'
        else:
            cmd += f'{self.error_handling}"'
        return cmd

    def _get_log_level_arg(self):
        """Return the correct log level argument for TonieToolbox CLI based on self.log_level."""
        level = str(self.log_level).strip().upper()
        if level == 'DEBUG':
            return '--debug'
        elif level == 'INFO':
            return '--info'
        return '--silent'

    def _setup_commands(self):
        """Set up all command strings for registry entries dynamically."""
        log_level_arg = self._get_log_level_arg()
        # Audio file commands
        self.convert_cmd = self._build_cmd(f'{log_level_arg}', log_to_file=self.log_to_file)
        self.upload_cmd = self._build_cmd(f'{log_level_arg}', use_upload=True, log_to_file=self.log_to_file)
        self.upload_artwork_cmd = self._build_cmd(f'{log_level_arg}', use_upload=True, use_artwork=True, log_to_file=self.log_to_file)
        self.upload_artwork_json_cmd = self._build_cmd(f'{log_level_arg}', use_upload=True, use_artwork=True, use_json=True, log_to_file=self.log_to_file)

        # .taf file commands
        self.show_info_cmd = self._build_cmd(log_level_arg, use_info=True, keep_open=True, log_to_file=self.log_to_file)
        self.extract_opus_cmd = self._build_cmd(log_level_arg, is_split=True, log_to_file=self.log_to_file)
        self.upload_taf_cmd = self._build_cmd(log_level_arg, use_upload=True, log_to_file=self.log_to_file)
        self.upload_taf_artwork_cmd = self._build_cmd(log_level_arg, use_upload=True, use_artwork=True, log_to_file=self.log_to_file)
        self.upload_taf_artwork_json_cmd = self._build_cmd(log_level_arg, use_upload=True, use_artwork=True, use_json=True, log_to_file=self.log_to_file)
        self.compare_taf_cmd = self._build_cmd(log_level_arg, use_compare=True, keep_open=True, log_to_file=self.log_to_file)

        # Folder commands
        self.convert_folder_cmd = self._build_cmd(f'{log_level_arg}', is_recursive=True, is_folder=True, log_to_file=self.log_to_file)
        self.upload_folder_cmd = self._build_cmd(f'{log_level_arg}', is_recursive=True, is_folder=True, use_upload=True, log_to_file=self.log_to_file)
        self.upload_folder_artwork_cmd = self._build_cmd(f'{log_level_arg}', is_recursive=True, is_folder=True, use_upload=True, use_artwork=True, log_to_file=self.log_to_file)
        self.upload_folder_artwork_json_cmd = self._build_cmd(f'{log_level_arg}', is_recursive=True, is_folder=True, use_upload=True, use_artwork=True, use_json=True, log_to_file=self.log_to_file)

    def _apply_config_template(self):
        """Apply the default configuration template if config.json is missing or invalid."""
        config_path = os.path.join(self.output_dir, 'config.json')
        if not os.path.exists(config_path):
            with open(config_path, 'w') as f:
                json.dump(CONFIG_TEMPLATE, f, indent=4)
            logger.debug(f"Default configuration created at {config_path}")
            return CONFIG_TEMPLATE
        else:
            logger.debug(f"Configuration file found at {config_path}")
            return self._load_config()

    def _load_config(self):
        """Load configuration settings from config.json"""
        config_path = os.path.join(self.output_dir, 'config.json')
        if not os.path.exists(config_path):
            raise FileNotFoundError(f"Configuration file not found: {config_path}")
        
        with open(config_path, 'r') as f:
            config = json.loads(f.read())
        
        return config

    def _setup_upload(self):
        """Set up upload functionality based on config.json settings"""
        # Always initialize authentication flags
        self.basic_authentication = False
        self.client_cert_authentication = False
        self.none_authentication = False
        config = self.config
        try:            
            upload_config = config.get('upload', {})            
            self.upload_urls = upload_config.get('url', [])
            self.ignore_ssl_verify = upload_config.get('ignore_ssl_verify', False)
            self.username = upload_config.get('username', '')
            self.password = upload_config.get('password', '')
            self.basic_authentication_cmd = ''
            self.client_cert_cmd = ''
            if self.username and self.password:
                self.basic_authentication_cmd = f'--username {self.username} --password {self.password}'
                self.basic_authentication = True
            self.client_cert_path = upload_config.get('client_cert_path', '')
            self.client_cert_key_path = upload_config.get('client_cert_key_path', '')
            if self.client_cert_path and self.client_cert_key_path:
                self.client_cert_cmd = f'--client-cert {self.client_cert_path} --client-cert-key {self.client_cert_key_path}'
                self.client_cert_authentication = True
            if self.client_cert_authentication and self.basic_authentication:
                logger.warning("Both client certificate and basic authentication are set. Only one can be used.")
                return False
            self.upload_url = self.upload_urls[0] if self.upload_urls else ''
            if not self.client_cert_authentication and not self.basic_authentication and self.upload_url:
                self.none_authentication = True
            return bool(self.upload_url)
        except FileNotFoundError:
            logger.debug("Configuration file not found. Skipping upload setup.")
            return False
        except json.JSONDecodeError:
            logger.debug("Error decoding JSON in configuration file. Skipping upload setup.")
            return False
        except Exception as e:
            logger.debug(f"Unexpected error while loading configuration: {e}")
            return False

    def _reg_escape(self, s):
        """Escape a string for use in a .reg file (escape double quotes)."""
        return s.replace('"', '\\"')

    def _generate_audio_extensions_entries(self):
        """Generate registry entries for supported audio file extensions"""
        reg_lines = []
        for ext in SUPPORTED_EXTENSIONS:
            ext = ext.lower().lstrip('.')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell]')
            reg_lines.append('')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}]')
            reg_lines.append('"MUIVerb"="TonieToolbox"')
            reg_lines.append(f'"Icon"="{self.icon_path}"')
            reg_lines.append('"subcommands"=""')
            reg_lines.append('')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell]')
            # Convert
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\a_Convert]')
            reg_lines.append('@="Convert File to .taf"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\a_Convert\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.convert_cmd)}"')
            reg_lines.append('')
            if self.upload_enabled:
                # Upload
                reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\b_Upload]')
                reg_lines.append('@="Convert File to .taf and Upload"')
                reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\b_Upload\\command]')
                reg_lines.append(f'@="{self._reg_escape(self.upload_cmd)}"')
                reg_lines.append('')
                # Upload + Artwork
                reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\c_UploadArtwork]')
                reg_lines.append('@="Convert File to .taf and Upload + Artwork"')
                reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\c_UploadArtwork\\command]')
                reg_lines.append(f'@="{self._reg_escape(self.upload_artwork_cmd)}"')
                reg_lines.append('')
                # Upload + Artwork + JSON
                reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\d_UploadArtworkJson]')
                reg_lines.append('@="Convert File to .taf and Upload + Artwork + JSON"')
                reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}\\shell\\d_UploadArtworkJson\\command]')
                reg_lines.append(f'@="{self._reg_escape(self.upload_artwork_json_cmd)}"')
                reg_lines.append('')
        return reg_lines

    def _generate_taf_file_entries(self):
        """Generate registry entries for .taf files"""
        reg_lines = []
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell]')
        reg_lines.append('')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}]')
        reg_lines.append('"MUIVerb"="TonieToolbox"')
        reg_lines.append(f'"Icon"="{self.icon_path}"')
        reg_lines.append('"subcommands"=""')
        reg_lines.append('')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell]')
        # Show Info
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\a_ShowInfo]')
        reg_lines.append('@="Show Info"')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\a_ShowInfo\\command]')
        reg_lines.append(f'@="{self._reg_escape(self.show_info_cmd)}"')
        reg_lines.append('')
        # Extract Opus Tracks
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\b_ExtractOpus]')
        reg_lines.append('@="Extract Opus Tracks"')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\b_ExtractOpus\\command]')
        reg_lines.append(f'@="{self._reg_escape(self.extract_opus_cmd)}"')
        reg_lines.append('')
        if self.upload_enabled:
            # Upload
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\c_Upload]')
            reg_lines.append('@="Upload"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\c_Upload\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.upload_taf_cmd)}"')
            reg_lines.append('')
            # Upload + Artwork
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\d_UploadArtwork]')
            reg_lines.append('@="Upload + Artwork"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\d_UploadArtwork\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.upload_taf_artwork_cmd)}"')
            reg_lines.append('')
            # Upload + Artwork + JSON
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\e_UploadArtworkJson]')
            reg_lines.append('@="Upload + Artwork + JSON"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\e_UploadArtworkJson\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.upload_taf_artwork_json_cmd)}"')
            reg_lines.append('')
        # Compare TAF Files
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\f_CompareTaf]')
        reg_lines.append('@="Compare with another .taf file"')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}\\shell\\f_CompareTaf\\command]')
        reg_lines.append(f'@="{self._reg_escape(self.compare_taf_cmd)}"')
        reg_lines.append('')
        return reg_lines

    def _generate_folder_entries(self):
        """Generate registry entries for folders"""
        reg_lines = []
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell]')
        reg_lines.append('')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}]')
        reg_lines.append('"MUIVerb"="TonieToolbox"')
        reg_lines.append(f'"Icon"="{self.icon_path}"')
        reg_lines.append('"subcommands"=""')
        reg_lines.append('')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell]')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\a_ConvertFolder]')
        reg_lines.append('@="Convert Folder to .taf (recursive)"')
        reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\a_ConvertFolder\\command]')
        reg_lines.append(f'@="{self._reg_escape(self.convert_folder_cmd)}"')
        reg_lines.append('')
        if self.upload_enabled:
            # Upload    
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\b_UploadFolder]')
            reg_lines.append('@="Convert Folder to .taf and Upload (recursive)"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\b_UploadFolder\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.upload_folder_cmd)}"')
            reg_lines.append('')
            # Upload + Artwork
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\c_UploadFolderArtwork]')
            reg_lines.append('@="Convert Folder to .taf and Upload + Artwork (recursive)"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\c_UploadFolderArtwork\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.upload_folder_artwork_cmd)}"')
            reg_lines.append('')
            # Upload + Artwork + JSON
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\d_UploadFolderArtworkJson]')
            reg_lines.append('@="Convert Folder to .taf and Upload + Artwork + JSON (recursive)"')
            reg_lines.append(f'[HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}\\shell\\d_UploadFolderArtworkJson\\command]')
            reg_lines.append(f'@="{self._reg_escape(self.upload_folder_artwork_json_cmd)}"')
            reg_lines.append('')
        return reg_lines

    def _generate_uninstaller_entries(self):
        """Generate registry entries for uninstaller"""
        unreg_lines = [
            'Windows Registry Editor Version 5.00',
            '',
        ]
        
        for ext in SUPPORTED_EXTENSIONS:
            ext = ext.lower().lstrip('.')
            unreg_lines.append(f'[-HKEY_CLASSES_ROOT\\SystemFileAssociations\\.{ext}\\shell\\{self.cascade_name}]')
            unreg_lines.append('')
            
        unreg_lines.append(f'[-HKEY_CLASSES_ROOT\\SystemFileAssociations\\.taf\\shell\\{self.cascade_name}]')
        unreg_lines.append('')
        unreg_lines.append(f'[-HKEY_CLASSES_ROOT\\Directory\\shell\\{self.cascade_name}]')
        
        return unreg_lines
    
    def generate_registry_files(self):
        """
        Generate Windows registry files for TonieToolbox context menu integration.
        Returns the path to the installer registry file.
        """
        os.makedirs(self.output_dir, exist_ok=True)
        
        reg_lines = [
            'Windows Registry Editor Version 5.00',
            '',
        ]
        
        # Add entries for audio extensions
        reg_lines.extend(self._generate_audio_extensions_entries())
        
        # Add entries for .taf files
        reg_lines.extend(self._generate_taf_file_entries())
        
        # Add entries for folders
        reg_lines.extend(self._generate_folder_entries())
        
        # Write the installer .reg file
        reg_path = os.path.join(self.output_dir, 'tonietoolbox_context.reg')
        with open(reg_path, 'w', encoding='utf-8') as f:
            f.write('\n'.join(reg_lines))
        
        # Generate and write the uninstaller .reg file
        unreg_lines = self._generate_uninstaller_entries()
        unreg_path = os.path.join(self.output_dir, 'remove_tonietoolbox_context.reg')
        with open(unreg_path, 'w', encoding='utf-8') as f:
            f.write('\n'.join(unreg_lines))
        
        return reg_path
        
    def install_registry_files(self, uninstall=False):
        """
        Import the generated .reg file into the Windows registry with UAC elevation.
        If uninstall is True, imports the uninstaller .reg file.
        
        Returns:
            bool: True if registry import was successful, False otherwise.
        """
        import subprocess
        reg_file = os.path.join(
            self.output_dir,
            'remove_tonietoolbox_context.reg' if uninstall else 'tonietoolbox_context.reg'
        )
        if not os.path.exists(reg_file):
            logger.error(f"Registry file not found: {reg_file}")
            return False
    
        ps_command = (
            f"Start-Process reg.exe -ArgumentList @('import', '{reg_file}') -Verb RunAs -Wait -PassThru"
        )
        try:
            result = subprocess.run(["powershell.exe", "-Command", ps_command], check=False, 
                                   capture_output=True, text=True)
                        
            if result.returncode == 0:
                logger.info(f"{'Uninstallation' if uninstall else 'Installation'} registry import completed.")
                return True
            else:
                logger.error(f"Registry import command failed with return code {result.returncode}")
                logger.error(f"STDERR: {result.stderr}")
                return False
                
        except subprocess.SubprocessError as e:
            logger.error(f"Failed to import registry file: {e}")
            return False

    @classmethod
    def install(cls):
        """
        Generate registry files and install them with UAC elevation.
        """
        instance = cls()
        instance.generate_registry_files()
        if instance.install_registry_files(uninstall=False):
            logger.info("Integration installed successfully.")
            return True
        else:
            logger.error("Integration installation failed.")
            return False

    @classmethod
    def uninstall(cls):
        """
        Generate registry files and uninstall them with UAC elevation.
        """
        instance = cls()
        instance.generate_registry_files()
        if instance.install_registry_files(uninstall=True):
            logger.info("Integration uninstalled successfully.")
            return True
        else:
            logger.error("Integration uninstallation failed.")
            return False